'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultEvent = {
    ok: _utils2.default.noop,
    err: _utils2.default.noop,
    complete: _utils2.default.noop,
    mask: _utils2.default.noop,
    unmask: _utils2.default.noop,
    onUploadProgress: null,
    onDownloadProgress: null
};

var defaultConfig = {
    method: 'get',
    timeout: 10 * 1000,
    responseType: 'json',
    headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json' }
};

var defaultResult = {
    success: 'success',
    msg: 'msg',
    data: 'data',
    format: null
};

var merge = function merge(options) {
    var event = _utils2.default.applyIf({}, options, defaultEvent);
    var result = typeof options.result === 'function' ? { format: options.result } : _utils2.default.apply({}, options.result || {}, defaultResult);

    var config = _utils2.default.apply({}, {
        baseURL: (typeof window !== 'undefined' ? window.baseURL : '') || '',
        method: options.data ? 'post' : 'get'
    }, defaultConfig);

    var accessToken = options.token || (typeof window === 'undefined' ? null : window.token);
    if ((typeof accessToken === 'undefined' ? 'undefined' : (0, _typeof3.default)(accessToken)) === 'object') {
        _utils2.default.apply(config.headers, accessToken);
    } else if (typeof accessToken === 'string') {
        _utils2.default.apply(config.headers, { 'x-access-token': accessToken });
    }
    if (options.token) {
        delete options.token;
    }
    if (options.result) {
        delete options.result;
    }

    for (var key in options) {
        if (!event[key]) {
            config[key] = options[key];
        }
    }

    return { config: config, event: event, result: result };
};

var query = function query(options) {
    var _merge = merge(options),
        config = _merge.config,
        event = _merge.event,
        result = _merge.result;

    event.mask();
    (0, _axios2.default)(config).then(function (res) {
        event.unmask(config.mask);
        event.complete(res);
        try {
            if (res.status == 200) {
                if (result.format) {
                    event.ok(result.format(res.data));
                } else {
                    if (res.data[result.success]) {
                        event.ok(res.data[result.data], res.data[result.msg]);
                    } else {
                        if (res.data.expire && typeof window !== 'undefined' && window.onExpire) {
                            window.onExpire(res.data);
                        } else {
                            event.err(res.data);
                        }
                    }
                }
            } else {
                event.err({ success: false, msg: 'status=' + res.status, _raw: res.data });
            }
        } catch (e) {
            event.err({ success: false, msg: e.message || '数据处理出错', _raw: e });
        }
    }).catch(function (error) {
        event.unmask(config.mask);
        event.complete(error);
        event.err({ success: false, msg: '请求数据异常', _raw: error });
    });
};

var fetch = function fetch(options) {
    return new _promise2.default(function (resolve) {
        query(_utils2.default.apply(options, {
            ok: function ok(res, msg) {
                resolve({ success: true, msg: msg, data: res });
            },
            err: function err(res) {
                resolve(res);
            }
        }));
    });
};

exports.default = {
    query: query, fetch: fetch
};