'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function ($ele) {
    MessageBox = $ele.MessageBox, Message = $ele.Message, Loading = $ele.Loading;
    return cfg;
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MessageBox = void 0,
    Message = void 0,
    Loading = void 0,
    loadingInstance = void 0;

var cfg = {
    alert: function alert(title, content, onHide, html) {
        if (arguments.length == 0) {
            return;
        }
        if (arguments.length == 1) {
            content = title;
            title = '';
            onHide = function onHide() {};
        } else if (arguments.length == 2) {
            if (typeof content == 'function') {
                onHide = content;
                content = title;
                title = '';
            } else {
                onHide = function onHide() {};
            }
        }
        MessageBox.alert(content, title, {
            confirmButtonText: '确定',
            callback: function callback(action) {
                onHide();
            },
            dangerouslyUseHTMLString: html || false
        });
    },
    toast: function toast(type, text, onHide) {
        var duration = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 3000;

        onHide = onHide || function () {};
        return Message({
            type: type,
            message: text,
            duration: duration
        });
    },
    success: function success(text, onHide) {
        return this.toast('success', text, onHide);
    },
    error: function error(text, onHide) {
        return this.toast('error', text, onHide);
    },
    info: function info(text, onHide) {
        return this.toast('info', text, onHide);
    },
    confirm: function confirm(title, content, yes, no) {
        if (arguments.length == 1) {
            content = title;
            title = '';
            yes = function yes() {};
            no = function no() {};
        } else if (arguments.length == 2) {
            if (typeof content == 'function' || (typeof content === 'undefined' ? 'undefined' : (0, _typeof3.default)(content)) == 'object') {
                yes = content;
                no = function no() {};
                content = title;
                title = '';
            } else {
                yes = function yes() {};
                no = function no() {};
            }
        } else {
            no = no || function () {};
        }
        if (typeof yes == 'function') {
            yes = { text: '确定', handler: yes };
        }
        if (typeof no == 'function') {
            no = { text: '取消', handler: no };
        }
        MessageBox.confirm(content, title, {
            confirmButtonText: yes.text,
            cancelButtonText: no.text,
            type: 'warning',
            dangerouslyUseHTMLString: false
        }).then(function () {
            yes.handler();
        }).catch(function () {
            no.handler();
        });
    },
    mask: function mask(text) {
        loadingInstance = Loading.service({ text: text });
    },
    unmask: function unmask() {
        try {
            if (loadingInstance) {
                loadingInstance.close();
            }
        } catch (e) {
            console.log(e);
        }
    },
    prompt: function prompt(options, vm) {
        options = $.apply({}, options, {
            title: '提示', msg: '', error: '', type: 'text',
            regex: function regex(v) {
                return $.isEmpty(v) ? '不能为空' : true;
            },
            value: null, handler: function handler() {},
            loading: function loading(ing, instance) {
                vm.$set(instance, 'confirmButtonLoading', ing);
                if (ing) {
                    vm.$set(instance, 'confirmButtonText', '提交中...');
                } else if (!instance.confirmButtonLoading) {
                    instance.close();
                }
            }
        });
        if (typeof options.handler === 'function') {
            options.handler = { text: '确定', handler: options.handler };
        }

        var cfg = {
            confirmButtonText: options.handler.text,
            cancelButtonText: '取消',
            inputErrorMessage: options.error,
            inputValue: options.value,
            inputType: options.type,
            inputPlaceholder: options.emptyText || '',
            beforeClose: function beforeClose(action, instance) {
                if (action === 'confirm') {
                    options.handler.handler(instance.inputValue, function (ing) {
                        options.loading(ing, instance);
                    });
                } else if (!instance.confirmButtonLoading) {
                    instance.close();
                }
            }
        };
        if (options.regex) {
            if (typeof options.regex === 'function') {
                cfg.inputValidator = options.regex;
            } else {
                cfg.inputPattern = options.regex;
            }
        }
        MessageBox.prompt(options.msg || '-', options.title, cfg).then($.noop).catch($.noop);
    }
};