'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function (vux) {
    $vux = vux;
    return modal;
};

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var $vux = null;


var modal = {
    alert: function alert(title, content, onHide) {
        if (arguments.length == 0) {
            return;
        }
        if (arguments.length == 1) {
            content = title;
            title = '';
            onHide = function onHide() {};
        } else if (arguments.length == 2) {
            if (typeof content == 'function') {
                onHide = content;
                content = title;
                title = '';
            } else {
                onHide = function onHide() {};
            }
        }
        if ($vux.alert) {
            $vux.alert.show({
                title: title,
                content: content,
                onHide: onHide
            });
        }
    },
    toast: function toast(type, text, onHide) {
        onHide = onHide || function () {};
        if ($vux.toast) {
            $vux.toast.show({ text: text, type: type, onHide: onHide, position: 'top' });
        }
    },
    success: function success(text, onHide) {
        this.toast('success', text, onHide);
    },
    error: function error(text, onHide) {
        this.toast('warn', text, onHide);
    },
    info: function info(text, onHide) {
        this.toast('text', text, onHide);
    },
    confirm: function confirm(title, content, yes, no) {
        var setting = {};
        if ((typeof title === 'undefined' ? 'undefined' : (0, _typeof3.default)(title)) === 'object') {
            setting = title;
        } else {
            if (arguments.length === 1) {
                content = title;
            } else if (arguments.length === 2) {
                if (typeof content === 'function' || (typeof content === 'undefined' ? 'undefined' : (0, _typeof3.default)(content)) === 'object') {
                    yes = content;
                    no = _utils2.default.noop;
                    content = title;
                    title = '';
                }
            }
            if (!yes) {
                yes = _utils2.default.noop;
            }
            if (!no) {
                no = _utils2.default.noop;
            }
            setting = { title: title, content: content, yes: yes, no: no };
        }
        setting = _utils2.default.apply({}, setting, { title: '', content: '', yes: { text: '确定', handler: _utils2.default.noop }, no: { text: '取消', handler: _utils2.default.noop } });

        if (typeof setting.yes === 'function') {
            setting.yes = { text: '确定', handler: yes };
        }
        if (typeof setting.no === 'function') {
            setting.no = { text: '取消', handler: no };
        }

        if ($vux.confirm) {
            $vux.confirm.show({
                title: setting.title,
                content: setting.content,
                confirmText: setting.yes.text,
                cancelText: setting.no.text,
                onConfirm: setting.yes.handler,
                onCancel: setting.no.handler
            });
        }
    },
    mask: function mask(text) {
        if ($vux.loading) {
            $vux.loading.show({ text: text });
        }
    },
    unmask: function unmask() {
        if ($vux.loading) {
            $vux.loading.hide();
        }
    }
};