import axios from 'axios'
import _ from './utils'


const defaultEvent = {
    ok: _.noop,
    err: _.noop,
    complete: _.noop,
    mask: _.noop,
    unmask: _.noop,
    onUploadProgress: null,
    onDownloadProgress: null
}

const defaultConfig = {
    method: 'get',
    timeout: 10 * 1000,
    responseType: 'json',
    headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json' //'Content-Type': 'application/x-www-form-urlencoded'
    }
}

const defaultResult = {
    success: 'success',
    msg: 'msg',
    data: 'data',
    format: null
}

const merge = function (options) {
    let event = _.applyIf({}, options, defaultEvent)
    let result = (typeof options.result === 'function') ? { format: options.result } : _.apply({}, options.result || {}, defaultResult)

    let config = _.apply({}, {
        baseURL: ((typeof window !== 'undefined') ? window.baseURL : '') || '',
        method: (options.data) ? 'post' : 'get'
    }, defaultConfig);

    let accessToken = options.token || ((typeof window === 'undefined') ? null :window.token)
    if (typeof accessToken === 'object') {
        _.apply(config.headers, accessToken)
    } else if (typeof accessToken === 'string') {
        _.apply(config.headers, { 'x-access-token': accessToken })
    }
    if (options.token) {
        delete options.token
    }
    if (options.result) {
        delete options.result
    }

    for (let key in options) {
        if (!event[key]) {
            config[key] = options[key]
        }
    }


    return { config, event, result }
}

const query = function (options) {
    let { config, event, result } = merge(options);
    event.mask()
    axios(config).then(function (res) {
        event.unmask(config.mask)
        event.complete(res)
        try {
            if (res.status == 200) {
                if (result.format) {
                    event.ok(result.format(res.data))
                } else {
                    if (res.data[result.success]) {
                        event.ok(res.data[result.data], res.data[result.msg])
                    } else {
                        if(res.data.expire && typeof window !== 'undefined' && window.onExpire){
                            window.onExpire(res.data)
                        }else{
                            event.err(res.data)
                        }
                    }
                }
            } else {
                event.err({ success: false, msg: 'status=' + res.status, _raw: res.data })
            }
        } catch (e) {
            event.err({ success: false, msg: e.message || '数据处理出错', _raw: e })
        }
    }).catch(function (error) {
        event.unmask(config.mask)
        event.complete(error)
        event.err({ success: false, msg: '请求数据异常', _raw: error })
    });
}


const fetch = function (options) {
    return new Promise(resolve => {
        query(_.apply(options, {
            ok: (res, msg) => {
                resolve({ success: true, msg, data: res })
            },
            err: res => {
                resolve(res)
            }
        }))
    })
}

export default {
    query, fetch
}