import fns from 'date-fns'
import zh_cn from 'date-fns/locale/zh_cn'

const startsWith = function(string, target, position) {
    const { length } = string
    position = position == null ? 0 : position
    if (position < 0) {
        position = 0
    } else if (position > length) {
        position = length
    }
    target = `${target}`
    return string.slice(position, position + target.length) == target
}

var date = null
var $date = function(_date = new Date(), onlyday = false) {
    if (arguments.length === 1 && typeof arguments[0] === 'boolean') {
        onlyday = arguments[0]
        _date = new Date()
    }
    if (typeof _date === 'object' && _date._flag) {
        return _date
    }
    if (onlyday) {
        _date = fns.format(_date, 'YYYY-MM-DD')
    }
    date = fns.parse(_date)
}

for (let key in fns) {
    if (startsWith(key, 'add') || startsWith(key, 'sub')) {
        $date.prototype[key] = function(num) {
            date = fns[key](date, num)
            return this
        }
    } else if (startsWith(key, 'endOf') || startsWith(key, 'get') || startsWith(key, 'set') || startsWith(key, 'lastDayOf') || startsWith(key, 'startOf')) {
        $date.prototype[key] = function() {
            return fns[key](date)
        }
    } else if (startsWith(key, 'differenceIn')) {
        /**
         * 获取给定日期之间的日历相应数量
         */
        $date.prototype[key] = function(dirtyDateLeft, dirtyDateRight) {
            if (dirtyDateRight === undefined) {
                dirtyDateRight = dirtyDateLeft
                dirtyDateLeft = date
            }
            return fns[key](dirtyDateLeft, dirtyDateRight)
        }
    } else if (['closestIndexTo', 'closestTo'].indexOf(key) > -1) {
        /**
         * closestIndexTo  从数组中返回与给定的DA相比的最接近日期的索引
         * closestTo 从最接近给定日期的数组返回日期。
         */
        $date.prototype[key] = function(datesArray) {
            return fns[key](date, datesArray)
        }
    } else if (key === 'format') {
        $date.prototype[key] = function(dirtyFormatStr) {
            return fns[key](date, dirtyFormatStr, { locale: zh_cn })
        }
    } else if (startsWith(key, 'is')) {
        $date.prototype[key] = function() {
            if (arguments.length === 0) {
                return fns[key](date)
            } else if (arguments.length === 1) {
                return fns[key](date, arguments[0])
            } else {
                return fns[key](...arguments)
            }
        }
    } else {
        $date.prototype[key] = fns[key]
    }
}

$date.prototype.getTimeStamp = function() {
    return parseInt(date.getTime() / 1000)
}
$date.prototype.ymd = function() {
    date = fns.parse(date.format('YYYY-MM-DD'))
    return this
}
$date.prototype.getWeekRange = function(week) {
    let curr = fns.getISOWeek(date)
    let w = (week ? week : curr) - curr
    let start = fns.startOfISOWeek(fns.addWeeks(date, w))
    let end = fns.endOfISOWeek(fns.addWeeks(date, w))
    return { start, end }
}
$date.prototype.getMonthRange = function() {
    return {
        start: fns.startOfMonth(date),
        end: fns.endOfMonth(date)
    }
}

$date.prototype.toString = function() {
    return date
}
$date.prototype._flag = true


export default function() {
    return new $date(...arguments)
}