import _ from './utils'
import axios from 'axios'


export default class QiniuClient {
    constructor({ server = 'http://upload.qiniu.com', buckets = [], api, accessToken = '' }) {
        this.server = server
        this.buckets = buckets
        this.tokens = []
        this.api = api
        //访问服务器的token
        this.queryAccessToken = () => {
            if (typeof accessToken === 'string') {
                return accessToken
            } else if (typeof accessToken === 'function') {
                return accessToken()
            } else {
                return ''
            }
        }
    }
    url(bucket, key, style = '', params = '') {
        var item = this.buckets.find(a => a.name == bucket);
        if (item) {
            let url = item.uri + '/' + key;
            if (!_.isEmpty(style)) {
                url += '?' + style
            }
            if (!_.isEmpty(params)) {
                url += ((url.indexOf('?') == -1) ? '?' : '&') + params
            }
            return url
        } else {
            console.warn('没有找到' + bucket + '对应的url')
            return 'data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=='
        }
    }
    query(value, x_access_token) {
        return new Promise(resolve => {
            axios({
                url: this.api,
                data: value,
                method: 'post',
                timeout: 10 * 1000,
                responseType: 'json',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'x-access-token': x_access_token
                }
            }).then(res => {
                try {
                    if (res.status == 200) {
                        if (res.data.success) {
                            resolve(res.data.data)
                        } else {
                            console.warn('请求七牛token失败', res.data)
                            resolve({ value: '', expire: 0 })
                        }
                    } else {
                        console.warn('请求七牛token失败', res.data)
                        resolve({ value: '', expire: 0 })
                    }
                } catch (e) {
                    console.warn('请求七牛token失败', e)
                    resolve({ value: '', expire: 0 })
                }
            }).catch(error => {
                console.warn('请求七牛token失败', error)
                resolve({ value: '', expire: 0 })
            });
        })
    }
    file2base64(file) {
        return new Promise(resolve => {
            var reader = new FileReader();
            reader.onload = function (e) {
                //this.result就是base64的数据了
                resolve(this.result)
            };
            reader.readAsDataURL(file);
        })
    }
    /**
     * 可以传入一个Object {bucket,key,expires,body}
     * 可以只传入一个字符串，则bucket=传入值 key=''
     * 可以传入两个字符串，分别为bucket,key
     */
    async getToken() {
        let value = { bucket: '', key: '', expires: 3600, body: null }
        if (arguments.length === 1 && typeof arguments[0] === 'string') {
            value.bucket = arguments[0]
        } else if (arguments.length === 2 && typeof arguments[0] === 'string' && typeof arguments[1] === 'string') {
            value.bucket = arguments[0]
            value.key = arguments[1]
        } else if (arguments.length === 1 && typeof arguments === 'object') {
            value = _.applyIf(value, arguments[0])
        } else {
            console.warn('七牛token传参异常')
            return ''
        }
        let { bucket, key, expires, body } = value

        let old = this.tokens.find(a => a.id === (bucket + key))
        if (old) {
            if (old.expire > new Date().getTime()) {
                return old.value
            } else {
                this.tokens.remove(old)
            }
        }
        if (_.isEmpty(this.api)) {
            console.warn('没有提供tokenApi地址用于获取upToken')
            return ''
        }
        let token = await this.query(value, this.queryAccessToken())
        if (token.value) {
            this.tokens.push({ id: bucket + key, token })
            return token.value
        } else {
            return ''
        }

    }
    async uploadFile({bucket,key}, file, onProgress) {
        let token = await this.getToken(bucket,key)
        if (_.isEmpty(token)) {
            return { success: false, msg: '获取upToken失败' }
        }
        let config = {
            headers: { 'Content-Type': 'multipart/form-data' },
            onUploadProgress: function (evt) {
                if (onProgress && evt.total > 0) {
                    evt.percent = Math.round(evt.loaded / evt.total * 100)
                    onProgress(evt);
                }
            }
        }
        let param = new FormData(); //创建form对象
        param.append('chunk', '0'); //断点传输
        param.append('chunks', '1');
        if (key) {
            param.append('key', key)
        }
        param.append('file', file, file.name)
        param.append('token', token);
        return new Promise(resolve => {
            axios.post(this.server, param, config)
                .then(response => {
                    let res = null
                    if (response.status == 200) {
                        res = { success: true, data: response.data }
                    } else {
                        res = { success: false, msg: 'status=' + response.status, _raw: response.data }
                    }
                    resolve(res)
                }).catch(function (error) {
                    let res = { success: false, msg: '请求数据异常', _raw: error }
                    resolve(res)
                })
        })
    }
    async uploadBase64({ bucket, key }, data, onProgress) {
        data = data.replace(/^data:image\/(\w+);base64,/, '')
        var url = this.server + "/putb64/-1";
        if (_.isEmpty(key)) {
            return { success: false, msg: '必须有key值' }
        }
        url += '/key/' + _.base64.encode(key);
        if (typeof XMLHttpRequest === 'undefined') {
            alert('浏览器不支持上传')
            return { success: false, msg: '浏览器不支持上传' }
        }
        let token = await this.getToken(bucket,key)
       
        if (_.isEmpty(token)) {
            return { success: false, msg: '获取upToken失败' }
        }
        return new Promise((resolve, reject) => {
            var xhr = new XMLHttpRequest();

            if (xhr.upload && onProgress) {
                xhr.upload.onprogress = function (evt) {
                    if (onProgress && evt.total > 0) {
                        evt.percent = Math.round(evt.loaded / evt.total * 100)
                        onProgress(evt);
                    }
                }
            }
            xhr.onreadystatechange = function () {
                if (xhr.readyState == 4) {
                    if (xhr.status == 200) {
                        var res = {};
                        eval('res=' + xhr.responseText);
                        res = { success: true, data: res }
                        resolve(res)
                    } else {
                        var res = { success: false, msg: '保存失败', raw: xhr.responseText };
                        resolve(res)
                    }
                }
            }

            var contentType = 'application/octet-stream';
            if (key.indexOf('.jpg') > -1)
                contentType = 'image/jpeg';
            else if (key.indexOf('.png') > -1)
                contentType = 'image/png';
            else if (key.indexOf('.json') > -1)
                contentType = 'application/json';


            xhr.open("POST", url, true);
            xhr.setRequestHeader("Content-Type", contentType);
            xhr.setRequestHeader("Authorization", "UpToken " + token);
            xhr.send(data);
        })
    }
}