import compare from './compare';
import base64 from 'base-64';
import vtype from './vtype';
import date from './date';

let _idSeed = 0;
export default {
    compare,base64,vtype,date,
    id() {
        return 'netfere-' + (++_idSeed);
    },
    uuid(norep) {
        var value = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0,
                v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
        if (norep) {
            return value
        } else {
            return value.replace(new RegExp('-', 'gm'), '');
        }
    },
    br2ln(value) {
        return value.replace(new RegExp('<br>', 'gm'), '\n')
    },
    ln2br(value) {
        return value.replace(new RegExp('\n', 'gm'), '<br>')
    },
    cnmoney(value, cs) {
        return '￥' + (value / (cs || 100)).toFixed(2);
    },
    filesize(value) {
        var size = 0;
        if (value > 1024 * 1024)
            size = (value / 1024 / 1024).toFixed(1) + 'M';
        else if (value > 1024)
            size = (value / 1024).toFixed(1) + 'K';
        else
            size = value + 'B';
        return size;
    },
    noop() { return false },
    rnd(n, m,includeMax=false) {
        if (m < n) {
            [n, m] = [m, n]
        }
        if(includeMax){
            m=m+1
        }
        var num = Math.random() * (m - n) + n;
        num = parseInt(num, 10);
        return num
    },
    rndString(len = 8) {
        var $chars = 'ABCDEFGHJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz012345678';
        var maxPos = $chars.length;
        var pwd = '';
        for (let i = 0; i < len; i++) {
            pwd += $chars.charAt(Math.floor(Math.random() * maxPos));
        }
        return pwd;
    },
    sleep(time, callback) {
        time = time || 2000
        return new Promise(function (resolve, reject) {
            setTimeout(function () {
                if (typeof callback === 'function') {
                    callback()
                }
                resolve();
            }, time);
        })
    },
    getUrlQuery(name, Url) {
        var reg = new RegExp("(^|\\?|&)" + name + "=([^&#]*)(\\s|&|#|$)", "i"),
            url = Url || location.href;
        if (reg.test(url))
            return unescape(RegExp.$2.replace(/\+/g, " "));
        return "";
    },
    apply(object, config, defaults) {
        if (defaults) {
            this.apply(object, defaults);
        }
        if (object && config && typeof config === 'object') {
            for (let key in config) {
                object[key] = config[key];
            }
        }
        return object;
    },
    applyIf(object, config, defaults) {
        if (defaults) {
            this.apply(object, defaults);
        }
        for (let key in config) {
            if (object[key] !== undefined) {
                object[key] = config[key];
            }
        }
        return object;
    },
    applyNo(object, config, defaults) {
        if (defaults) {
            this.apply(object, defaults);
        }
        for (let key in config) {
            if (object[key] === undefined) {
                object[key] = config[key];
            }
        }
        return object;
    },
    isEmpty(value, zero = true) {
        return value === null || value === undefined || value === false ||
            (typeof value === 'string' && value === '') ||
            (typeof value === 'number' && value === 0 && zero) ||
            (typeof Array.isArray(value) && value.length === 0) ||
            (typeof value === 'object' && Object.keys(value).length === 0)
    },
    deepCopy(source) {
        if (this.isEmpty(source)) {
            return source
        }
        if (Array.isArray(source)) {
            var result = []
            source.forEach(item => {
                result.push(this.deepCopy(item))
            })
            return result;
        } else if (typeof source === 'object') {
            var result = {};
            for (var key in source) {
                result[key] = typeof source[key] === 'object' ? this.deepCopy(source[key]) : source[key];
            }
            return result;
        } else {
            return source
        }
    },
    trim(value) {
        return value.replace(/\s*/g, '')
    },
    replaceHtml(html){
        return html.replace(/\s*/g, '').replace(/<[^>]*>/g,'')
    },
    truncation(content,len){
        if(content.length<len){
            return content
        }else{
            content.substr(0,len)+'...'
        }
    },
    has(object, key) {
        return object != null && Object.prototype.hasOwnProperty.call(object, key)
    },
    getChange(newValue, oldValue) {
        let value = {}
        for (let key in newValue) {
            if (!compare(newValue[key], oldValue[key])) {
                value[key] = newValue[key]
            }
        }
        return value
    },
    /**
     * 当fields为一项，且simple=true时，返回数组值
     * @param {*} array 
     * @param {*} fields 
     * @param {*} simple 
     */
    getFields(array, fields, simple = true) {
        if ($.isEmpty(array)) {
            return []
        }
        if (typeof fields === 'string') {
            fields = [fields]
        }
        let results = []
        array.forEach(rec => {
            let item = {}
            fields.forEach(key => {
                if (this.has(rec, key)) {
                    item[key] = rec[key]
                }
            })
            if(!this.isEmpty(item)){
                results.push(item)
            }
        })
        if (simple && fields.length === 1) {
            let arr = []
            results.forEach(item => {
                let value = item[fields[0]]
                if (!arr.exists(value)) {
                    arr.push(value)
                }
            })
            return arr
        } else {
            return results
        }
    },
    init() {
        Array.prototype.exists = function(item) {
            if (typeof item === 'function') {
                return this.find(item) !== undefined
            } else {
                return this.indexOf(item) > -1;
            }
        }

        Array.prototype.remove = function(query) {
            if (typeof query === 'function') {
                const result = []
                if (this.length === 0) {
                    return result
                }
                let length = this.length
                while (length--) {
                    let value = this[length]
                    if (query(value)) {
                        result.insert(0, value)
                        this.splice(length, 1)
                    }
                }
                return result
            } else if (typeof query === 'object' || Array.isArray(query)) {
                let idx = this.indexOf(query)
                if (idx > -1) {
                    this.splice(idx, 1)
                    return [query]
                } else {
                    return this.remove(a => compare(a, query))
                }
            } else {
                return this.remove(a => a === query)
            }
        }

        Array.prototype.insert = function(idx, item) {
            this.splice(idx, 0, item);
            return this
        }

        Date.prototype.format = function(dirtyFormatStr, dirtyOptions) {
            return date(this).format(dirtyFormatStr, dirtyOptions)
        }

        Date.prototype.toString = function(type) {
            type = type || 'full';
            var o = {
                date: 'YYYY-MM-DD',
                time: 'hh:mm:ss'
            }
            type = o[type] || 'YYYY-MM-DD HH:mm:ss';
            return this.format(type);
        }

        String.prototype.replaceAll = function(s1, s2) {
            return this.replace(new RegExp(s1, "gm"), s2);
        }

        String.prototype.trim=function(){
            return this.replace(/\s*/g, '')
        }
    },
    //advance提前多少时间，单位秒
    getToken(key,advance=0) {
        try {
            let obj = JSON.parse(window.localStorage.getItem(key))
            if (obj && obj.expire > new Date().getTime()+advance*1000) {
                return obj.token
            } else {
                return null
            }
        } catch (e) {
            return null
        }
    },
    setToken(key, value) {
        window.localStorage.setItem(key, JSON.stringify(value))
    },
    loadScript(url, callback){
        var script = document.createElement("script")
        script.type = "text/javascript";
        if (script.readyState) { //IE
            script.onreadystatechange = function () {
                if (script.readyState == "loaded" || script.readyState == "complete") {
                    script.onreadystatechange = null;
                    callback();
                }
            };
        } else { //Others
            script.onload = function () {
                callback();
            };
        }
        script.src = url;
        document.getElementsByTagName("head")[0].appendChild(script);
    }
}