import _ from '../utils'
import sdk from 'weixin-js-sdk'

if (sdk.hideOptionMenu) {
    sdk.hide = sdk.hideOptionMenu
}

if (sdk.closeWindow) {
    sdk.close = sdk.closeWindow
}

const getApiList = () => {
    let apis = {
        menu: ['hideOptionMenu', 'showOptionMenu', 'hideMenuItems', 'showMenuItems', 'hideAllNonBaseMenuItem', 'showAllNonBaseMenuItem'],
        share: ['onMenuShareTimeline', 'onMenuShareAppMessage', 'onMenuShareQQ', 'onMenuShareWeibo', 'onMenuShareQZone'],
        voice: ['startRecord', 'stopRecord', 'onVoiceRecordEnd', 'playVoice', 'pauseVoice', 'stopVoice', 'onVoicePlayEnd', 'uploadVoice', 'downloadVoice', 'translateVoice'],
        image: ['chooseImage', 'previewImage', 'uploadImage', 'downloadImage', 'getLocalImgData'],
        card: ['addCard', 'chooseCard', 'openCard'],
        pay: ['chooseWXPay', 'openProductSpecificView', 'openAddress'],
        base: ['closeWindow', 'getNetworkType', 'scanQRCode'],
        location: ['openLocation', 'getLocation']
    }
    let arr = []
    for (let key in apis) {
        apis[key].forEach(item => { arr.push(item) })
    }
    
    return arr
}

sdk.init = function(config) {
    sdk.error = res => {
        console.warn(res)
    }
    config = _.apply({}, config, {
        beta: true,
        debug: false,
        jsApiList: getApiList()
    });

    ['appId', 'timestamp', 'nonceStr', 'signature'].forEach(key => {
        if (!config[key]) {
            console.warn('微信jssdk初始化缺少 ' + key + ' 参数')
        }
    });
    sdk.ready(()=>{
        if(typeof config.init==='function'){
            config.init(sdk)
        }
    })
    sdk.config(config)
    
    return
    sdk.checkJsApi({
        jsApiList: config.jsApiList,
        success: res => {
            sdk.checkResult = res.checkResult || null
        }
    })
}
/**
 * 尝试获取照片的base64数据
 * localId为微信选择照片后本地信息
 * 返回数据有情况为
 * {id:localId,src:any}
 * src可能等于id(即localId),可能为base64数据
 * 
 * ios版本可能需要用window.__wxjs_is_wkwebview来判断使用wx.getLocalImgData
 */
const getLocalImgData = (localId) => {
    return new Promise(resolve => {
        try{
            sdk.getLocalImgData({
                localId,
                success: res => {
                    if (!res.localData) {
                        resolve({ id:localId, src: localId })
                        return
                    }
                    let img = res.localData
                    if (!img.startsWith('data:image')) {
                        img = 'data:image/jpeg;base64,' + img
                    } else {
                        img = img.replace('jgp', 'jpeg')
                    }
                    resolve({ id:localId, src: img })//此时为base64数据
                },
                error: e => {
                    resolve({ id:localId, src: localId })
                }
            })
        }catch(e){
            resolve({ id:localId, src: localId })
        }
    })
}
/**
 * 调用微信选择相册中的照片
 * 返回数据是[{id:'本地localid',src:'可能=id,可能为base64图片数据'}]
 */
sdk.getImage = function({ count=9, sizeType=['compressed', 'original'], sourceType=['album', 'camera'], success=_.noop, fail=_.noop, complete=_.noop }) {
    return new Promise(resolve => {
        let photos=[]
        sdk.chooseImage({
            count,
            sizeType,
            sourceType,
            success: async(res) => {
                for (let i = 0; i < res.localIds.length; i++) {
                    photos.push(await getLocalImgData(res.localIds[i]))
                }
                resolve(photos)
            },
            fail:res=>{
                resolve([])
            },
            complete
        })
    })
}
/**
 * 上传照片或语音到微信
 * 返回{success:true|false,msg:'当success=false时的错误',media_id:当success=true时，微信返回的媒体值}
 */
sdk.upload=function({ type='photo', mask=false, localId }){
    let isShowProgressTips = mask?1:0
    return new Promise(resolve => {
        try{
            if (type === 'voice') {
                sdk.uploadVoice({
                    localId,
                    isShowProgressTips,
                    success: function(res) {
                        resolve({ success: true, media_id: res.serverId })
                    },
                    fail: res => {
                        resolve({ success: false, msg: '上传失败' })
                    }
                })
            }else if(type==='photo'){
                sdk.uploadImage({
                    localId,
                    isShowProgressTips,
                    success: function(res) {
                        resolve({ success: true, media_id: res.serverId })
                    },
                    fail: function() {
                        resolve({ success: false, msg: '上传失败' })
                    }
                })
            }
        }catch(e){
            resolve({ success: false, msg: '上传失败'+e.toString() })
        }
    })
    
}

export default {
    install(Vue) {
        Vue.prototype.$wechat = sdk
        Vue.$wechat = sdk
    }
}